/**
 * com1553 - MIL-1553 Communication Library for Herschel - Initialization of RT.
 *
 * Filename             : \file init1553.c
 *
 * Purposes             : \brief [DONE] com1553 - MIL-1553 Communication Library for Herschel - Initialization of RT.
 *
 * Logical Task         : in Spire - INIT
 *                      : in Pacs  - TBW - TODO
 *                      : in HIFI  - TBW - TODO
 *                      : \ingroup group_COM1553
 *
 * Author               : Scige
 *
 * Last Developer       : $Author: lorenzo $
 *
 * Revision             : $Revision: 1.11 $
 *
 * Checkout Tag         : $Name:  $
 *
 * Last Modification    : $Date: 2006/10/20 09:22:14 $
 *
 * Location             : $RCSfile: init1553.c,v $
 *
 * \version             : $Header: /home/local/cvsrep/OBS_FM/1553_Lib/init1553.c,v 1.11 2006/10/20 09:22:14 lorenzo Exp $
 */

/*
 * Commitments History :
 * As reported in Main cvs Documentation
 * ( https://www.cvshome.org/docs/manual/cvs-1.11.18/cvs_12.html#SEC102 )
 * The Modification Log has been posted at End Of File.
 */
// ---------------------------------------------------------------------------//


// ---------------------------------------------------------------------------//
#include <string.h>

#include	"conf1553.h"

#if OBSCODE == HIFI_CODE

#elif OBSCODE == SPIRE_CODE

#elif OBSCODE == PACS_CODE
    #include	"1553_def.h"
    #include	"LT_TMdef.h"
#endif

#include	"init1553.h"
// --
#include	"ivar1553.h"

#include "MilConf.h"
#include "MilInit.h"
// ---------------------------------------------------------------------------//

// ---------------------------------------------------------------------------//
// -- Functions in this module
void main_1553_init ( void );
void main_1553_exit ( void );
static void dpu_rt_init ( void );
// ---------------------------------------------------------------------------//
// ---------------------------------------------------------------------------//
/**  void main_1553_init ( void );
 * \brief Initialize Packet Transfer Request Queue.\n
 * \brief Initialize DDC1553 Hardware Chip.\n
 * \brief Nest Interrupt Activation Chain in Virtuoso Environment.\n
 *
 * \note In Spire mode there is no Interrupt Activation Chain Nesting.
 * \note In Spire mode there is no registration of event handler.
 * \note In Spire mode there is no activation of event handler.
 * \note In Spire mode there is no registration of interrupt service.
 *
 * \callgraph
 * \ingroup group_COM1553
 */ 
void main_1553_init ( void )
{
	int gi;

	{
	    TmWriter=&TM_PACK[0];
	    TmReader=&TM_PACK[0];
	    for ( gi=0; gi<TM_PACK_REQUEST_NUM; gi++)
	    {
			TM_PACK[gi].tmreq	= 0x0000;
			TM_PACK[gi].count	= 0x0000;
	        TM_PACK[gi].status  = ISFREE;
	        TM_PACK[gi].next    = &TM_PACK[1+gi];
			memset(	TM_PACK[gi].offset, 0, 16 );	// nino 27/06/2003
	    }

	    TM_PACK[TM_PACK_REQUEST_NUM-1].next=&TM_PACK[0];
	}

	/// DDC 1553 - Forcing a soft reset.
	{
		// -- Start RT Operations
    	memset( (int*)0x8F004002, 0xFFFFFFFF, 0x00000007 );
	    memset( (int*)0x8F000000, 0x00000000, 0x00004000 );
	}

	/// Start RT Operations
	{
		dpu_rt_init();
	}

	readCmndDPRAM ( );

    FreeCmndDPRAM = MaxCmndDPRAM;
    FreePackDPRAM = MaxPackDPRAM;


#if OBSCODE == HIFI_CODE
    KS_EventSetHandler(ISR_1553_EVENT, isr1553);
    KS_EventEnable(ISR_1553_EVENT);
    KS_IRQSetHandler(6, irq2);
    KS_ISREnable(6);
#elif OBSCODE == SPIRE_CODE

#elif OBSCODE == PACS_CODE
    KS_EventSetHandler(ISR_1553_EVENT, isr1553);
    KS_EventEnable(ISR_1553_EVENT);
    KS_IRQSetHandler(6, irq2);
    KS_ISREnable(6);
#endif
}
// ---------------------------------------------------------------------------//

// ---------------------------------------------------------------------------//
/**  void main_1553_exit ( void )
 * \brief UNUSED - Clean up Communication Software Structures and DDC1553 Hardware Chip.

 *
 * \callgraph
 * \ingroup group_COM1553
 */
void main_1553_exit ( void )
{
	int i;
	MilReleaseHandle( MilRTConf, Tx_data_han27);
	MilReleaseHandle( MilRTConf, Rx_data_han27);
	
	for(i=0;i<4;i++)
		MilReleaseHandle( MilRTConf, Rx_data_han[i]);
	MilReleaseHandle( MilRTConf, Rx_data_han10);
	MilReleaseHandle( MilRTConf, Tx_data_han10);
	
	for(i=0;i<16;i++)
		MilReleaseHandle( MilRTConf, Tx_data_han[i]);
		
	//MilReleaseHandle( MilRTConf, Bcst_data_han);
	
	MilRTClose(MilRTConf);
	MilClose(MilRTConf);
}
// ---------------------------------------------------------------------------//

// ---------------------------------------------------------------------------//
/**  static void dpu_rt_init ( void )
 *
 * \brief DDC1553 Hardware Chip Initialization and Configuration of Communication Structures.
 *
 * \par Actual Configuration in the three systems:
 * \li Configure as REMOTE TERMINAL
 * \li Configure Sub Addresses [01-31][Tx/Rx] as Single Message no Circular buffer or Double Buffer.
 * \li Configure Sub Addresses [11-27][Tx/__] used for Telemetry as Circular buffer, of 128 Word.
 * \li Configure Sub Addresses [30   ][Tx/Rx] as LoopBack In DPRAM.
 * \li Configure Mode Command allowing Synchronize with and without data-word.
 *
 * \callgraph
 * \ingroup group_COM1553
 */ 
static void dpu_rt_init ( void )
{
    int i;

    MilRTConf = MilOpen();          /*sets up software library*/
    MilRTOpen(MilRTConf);     /* open rt mode for access */
    RTAddress = MilRTAddress(MilRTConf);     /* read the current rt address */
    MilRTDefMsgIllegal(MilRTConf,ALL,ALL,ALL);/*define all messages illegal*/

#if OBSCODE == HIFI_CODE
    ICU_Prime_Redundant =  ( RTAddress != ICU_PRIME );

#elif OBSCODE == SPIRE_CODE

    Dpu_Prime_Redundant =  ( RTAddress != DPU_PRIME );
    OBS_Vers = SPIRE_OBS_VER;
#elif OBSCODE == PACS_CODE

#endif

    sa_conf.BcstEomInt = FALSE;
    sa_conf.RxEomInt   = FALSE;
    sa_conf.TxEomInt   = FALSE;
    sa_conf.BcstBuffInt = FALSE;
    sa_conf.RxBuffInt   = FALSE;
    sa_conf.TxBuffInt   = FALSE;
    sa_conf.BcstMm = SINGLE_MESSAGE;
    sa_conf.RxMm   = SINGLE_MESSAGE;
    sa_conf.TxMm   = SINGLE_MESSAGE;
    sa_conf.RcvBufferType = SINGLEBUFFER;

    for (i=0; i<32; i++)
        MilRTDefSA(MilRTConf, i, &sa_conf);

// superseeds default configuration for Telemetry SAs where Circular Buffer are going to be used
    circ_sa_conf.BcstEomInt = FALSE;
    circ_sa_conf.RxEomInt   = FALSE;
    circ_sa_conf.TxEomInt   = FALSE;
    circ_sa_conf.BcstBuffInt = FALSE;
    circ_sa_conf.RxBuffInt   = FALSE;
    circ_sa_conf.TxBuffInt   = FALSE;
    circ_sa_conf.BcstMm = SINGLE_MESSAGE;
    circ_sa_conf.RxMm   = SINGLE_MESSAGE;
    circ_sa_conf.TxMm   = RTBUFFER128;
    for (i=11; i<27; i++)
		MilRTDefSA(MilRTConf, i, &circ_sa_conf);

// This section sets up transmit and receive memory block handlers for the different SubAddresses as needed
// Handlers are then mapped to physical blocks on the 1553 RAM
    Tx_data_han27 = MilRTAllocBlk(MilRTConf, SINGLE_MESSAGE);
    Rx_data_han27 = MilRTAllocBlk(MilRTConf, SINGLE_MESSAGE);

    for (i=0; i<4; i++)
		Rx_data_han[i] = MilRTAllocBlk(MilRTConf, SINGLE_MESSAGE);

    Rx_data_han10 = MilRTAllocBlk(MilRTConf, SINGLE_MESSAGE);
    Tx_data_han10 = MilRTAllocBlk(MilRTConf, SINGLE_MESSAGE);

    for (i=0; i<16; i++)
        Tx_data_han[i] = MilRTAllocBlk(MilRTConf, RTBUFFER128);

    //Bcst_data_han = MilRTAllocBlk(MilRTConf, sa_conf.BcstMm);
    //MilRTMapBlk(MilRTConf, 31, BROADCAST, Bcst_data_han, SINGLE_MESSAGE);

	Tx_data_han1 = MilRTAllocBlk(MilRTConf, SINGLE_MESSAGE);
	MilRTMapBlk(MilRTConf, 1, TRANSMIT, Tx_data_han1, SINGLE_MESSAGE);

    Tx_data_han8 = MilRTAllocBlk(MilRTConf, SINGLE_MESSAGE);// nino 22/09/2004
    MilRTMapBlk(MilRTConf, 8, TRANSMIT, Tx_data_han8, 0); // nino 22/09/2004

    Rx_data_han8 = MilRTAllocBlk(MilRTConf, SINGLE_MESSAGE);
    MilRTMapBlk(MilRTConf, 8, RECEIVE,  Rx_data_han8, 0);

	TRx_data_han30= MilRTAllocBlk(MilRTConf, SINGLE_MESSAGE);
	MilRTMapBlk(MilRTConf,30,  RECEIVE, TRx_data_han30, SINGLE_MESSAGE);
	MilRTMapBlk(MilRTConf,30, TRANSMIT, TRx_data_han30, SINGLE_MESSAGE);

    for (i=0; i<16; i++)
		MilRTMapBlk(MilRTConf, 11+i, TRANSMIT, Tx_data_han[i], 0);

    MilRTMapBlk(MilRTConf, 10, TRANSMIT, Tx_data_han10, 0);
    MilRTMapBlk(MilRTConf, 10, RECEIVE,  Rx_data_han10, 0);

	for (i=0; i<4; i++)
		MilRTMapBlk(MilRTConf, 11+i, RECEIVE, Rx_data_han[i], 0);
    MilRTMapBlk(MilRTConf, 27, TRANSMIT, Tx_data_han27, 0);
    MilRTMapBlk(MilRTConf, 27, RECEIVE,  Rx_data_han27, 0);

    ////////////////////////////////////////
    ////////////////////////////////////////

	MilRTDefMsgLegal	(MilRTConf,ALL, ALL, ALL);	//define all messages legal
	MilIrqAutoClear 	(MilRTConf,TRUE); // Interrupt Requests on irq2 (1553) are automatically cleared after being served

//	MilIrqDisable		(MilRTConf,IRQ_ALL); /* Int_Mask completed */
	MilIrqEnable		(MilRTConf,IRQ_STATUS_SET_MODE_INT_TRIG); // Enable interrupts on reception of mode codes

	MilRTEnhModeCode	(MilRTConf, TRUE);	// Configure 1553 to Enhanced mode code (where it is possible to raise interrupts upon mode code reception)
	MilRTModeCode		(MilRTConf, TRUE, FALSE);  // Configure 1553 for Advance mode codes
	// This was the bad setting that didn't with the mode code Synchronize Interrupt
//	  MilRTModeIrqEnable  (MilRTConf,TRUE,TRANSMIT,FALSE,RTMIRQ_SYNCHRONIZE);
//	  MilRTModeIrqEnable  (MilRTConf,TRUE,BROADCAST,TRUE,RTMIRQ_SYNCHRONIZE);
	MilRTModeIrqEnable	(MilRTConf, TRUE, TRANSMIT, FALSE, RTMIRQ_SYNCHRONIZE); // a SYNC_NO_DATA mode command will trigger an interrupt
	MilRTModeIrqEnable	(MilRTConf, TRUE,  RECEIVE,  TRUE, RTMIRQ_SYNCHRONIZE); // a SYNC_DATA mode command will trigger an interrupt

//	  MilIrqReset	  (MilRTConf);
	MilBlockFill	(MilRTConf,STACK_A, 0, 257); // Reset 1553 messages Stack Pointer - ninoOK 31/07/02 256 -> 257 to cover the stack pointer too.

	MilRTEnhMM(MilRTConf, TRUE); //RT Enhanced memory management capabilities
	MilRTAltStatusEna(MilRTConf, FALSE); //No alternate Status word - compliant to 1553B
	MilRTFlagWrap(MilRTConf, TRUE);  // it there is an RT faileure this is notified in the status word

	/* no flags set in status */
	MilRTSetBusy(MilRTConf, FALSE); // set BUSY bit to 0 - use of this bit is optional and we do not use here
	MilRTSetSSflag(MilRTConf, FALSE); // set SubSystem flag to 0 - use of this bit is optional and we do not use here
	MilRTSetSvcReq(MilRTConf, FALSE); // set service request flag to 0 - use of this bit is optional and we do not use here
	MilRTFlag(MilRTConf, FALSE); // set RTFLAG to 0 - toggling to 1 only if there is an RT failure (see above)

	MilRTRun(MilRTConf); // Starts RT
//    MilWriteReg(MilRTConf, CONFIG_1, 0x8F80);
}
// ---------------------------------------------------------------------------//


// ---------------------------------------------------------------------------//
/*
 * Commitments History :
 * $Log: init1553.c,v $
 * Revision 1.11  2006/10/20 09:22:14  lorenzo
 * 1553 library updated
 *
 * Revision 1.8  2006/10/16 14:19:53  scige
 * Some More Comments
 *
 * Revision 1.7  2006/07/04 10:08:08  lorenzo
 * restored handling of redundant unit: changed only within if code=HIFI_code, no effects for the other two obss
 *
 * Revision 1.9  2006/06/13 15:51:08  lorenzo
 * obs4 first branch merged
 *
 * Revision 1.8.4.1  2006/06/13 09:06:25  lorenzo
 * libs updated + error corrected
 *
 * Revision 1.6  2006/06/07 11:44:56  scige
 * Some More Comment
 * by Scige John Liu IFSI
 *
 * Revision 1.5  2006/04/27 10:21:44  scige
 * Some more Comment alignement before STRONG COMMENTING
 *
 * Revision 1.4  2006/02/15 13:06:56  scige
 * Comment Style Restored
 *
 * Revision 1.4  2006/02/10 14:48:44  scige
 * Backup some comment lost and some history log.
 * Introdusing new naming.
 * Introducing new file for a portable inclusion: iext1553.h.
 *
 * Revision 1.5  2006/02/09 11:23:59  scige
 * some var had 2 names
 *
 * Revision 1.4  2006/02/09 08:18:13  scige
 * Some Functions Renamed
 *
 * Revision 1.3  2006/02/08 15:23:52  scige
 * Better Tag Documentation
 *
 * Revision 1.2  2006/02/08 10:55:53  scige
 * Partitioning - "init1553.h" TO "init1553.h" AND "iext1553.h"
 *
 * Revision 1.1  2006/02/08 10:46:30  scige
 * Some minor Refinement
 *
 * Revision 1.1  2006/02/07 14:04:03  scige
 * New Directory Restiling
 *
 * Revision 1.1  2006/02/03 16:31:36  scige
 * Mil1553 Communication Library
 * New Base Commitment
 *
 * Revision 1.4  2005/09/16 13:09:40  scige
 * *** empty log message ***
 *
 * Revision 1.2  2005/08/26 12:16:19  scige
 * storage
 *
 * Revision 1.1  2005/08/10 12:53:39  scige
 * Communication Library: 1553
 * com1553
 * Base Common Version of Spire/Pacs OBS
 *
 *
 */

